import * as React from "react";
import Highlighter from "react-highlight-words";
import terms from "./terms.json";
import { KeyTerm } from "./KeyTerm";

export const HighlightKeyTerms: React.FC<{ children: React.ReactNode }> = ({
  children,
}) => {
  return (
    <>
      {React.Children.map(children, (child) => {
        if (typeof child !== "string") {
          return child;
        }
        return (
          <Highlighter
            searchWords={Object.keys(terms)}
            textToHighlight={child}
            findChunks={({ searchWords, textToHighlight }) => {
              const chunks = [];
              const normalizedTextToHighlight = textToHighlight.toLowerCase();

              for (const word of searchWords) {
                if (typeof word !== "string") {
                  continue;
                }

                if (!word.length) {
                  continue;
                }

                const index = normalizedTextToHighlight.indexOf(
                  word.toLowerCase()
                );

                if (index > -1) {
                  const term = terms[word.toLowerCase() as keyof typeof terms];

                  if (
                    term &&
                    "surroundingContextsToAvoid" in term.frontmatter &&
                    term.frontmatter.surroundingContextsToAvoid
                  ) {
                    // Escape characters for use in regexp
                    const escapedContexts =
                      term.frontmatter.surroundingContextsToAvoid.map(
                        (c: string) =>
                          c.replace(/[/\-\\^$*+?.()|[\]{}]/g, "\\$&")
                      );
                    if (
                      new RegExp(escapedContexts.join("|"), "i").test(
                        normalizedTextToHighlight
                      )
                    ) {
                      continue;
                    }
                  }
                  chunks.push({
                    start: index,
                    end: index + word.length,
                  });
                }
              }

              return chunks;
            }}
            highlightTag={({ children }) => {
              const term = terms[children.toLowerCase() as keyof typeof terms];

              if (!term) {
                console.warn(`No definition found for ${children}`);
                return null;
              }

              return <KeyTerm term={children} definition={term.definition} />;
            }}
          />
        );
      })}
    </>
  );
};
